
# q-expansion of newform 1428.2.a.h, downloaded from the LMFDB on 16 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1428
weight = 2
poly_data = [-10, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [1, 0], [1, 1], [1, 0], [3, 0], [3, 1], [-1, 0], [-5, -1], [1, -2], [4, -2], [-2, -1], [-6, 1], [-7, -1], [3, -2], [2, -1], [0, -3], [10, 1], [0, 1], [-10, 0], [2, -4], [6, -2], [4, -1], [6, 0], [-4, 2], [4, -4], [0, 3], [-9, 1], [5, 2], [6, 1], [-13, 2], [-13, 0], [-5, 1], [16, 1], [2, 6], [14, 2], [6, -2], [9, -5], [6, 4], [7, 1], [-7, 5], [-6, 6], [-14, 2], [6, 0], [-4, -3], [3, 0], [8, -3], [-16, -3], [-11, -1], [-7, 5], [-4, -6], [-1, 2], [10, 1], [12, 4], [-12, -3], [12, -3], [-4, 2], [9, 3], [-17, -1], [2, 6], [-20, -2], [-2, 5], [24, 0], [-12, 4], [-8, 4], [-20, -1], [2, -4], [-5, 2], [-4, -9], [-6, -6], [3, -5], [6, -3], [-10, -7], [-8, -4], [12, 4], [-12, 4], [4, 1], [0, 6], [-4, 9], [-3, 0], [5, -9], [-10, -4], [-11, 8], [24, 0], [5, -3], [-16, 3], [4, 1], [-12, 0], [27, 4], [-18, -3], [14, 6], [-2, -2], [7, 1], [-18, 4], [18, 0], [4, -1], [3, 9], [-8, 4], [-3, -3], [-20, 2], [-4, -6], [14, -3], [16, 1], [6, 6], [0, -6], [4, 8], [-27, -5], [-28, 5], [6, 3], [34, 4], [18, 7], [18, -5], [47, 0], [24, 6], [6, -5], [25, -4], [-7, 2], [-4, 0], [-10, -10], [1, -8], [24, 3], [-5, -7], [6, 4], [-19, 5], [9, -12], [-16, -3], [-28, 2], [30, 4], [-1, -1], [-8, 2], [-26, 2], [25, 2], [26, 2], [-8, 5], [7, -4], [18, 0], [-15, -5], [36, -3], [12, -11], [36, 0], [-7, 2], [-30, 4], [-19, 8], [-14, -1], [-27, 6], [14, 0], [-3, 3], [-10, -15], [-10, 14], [-20, 2], [-6, -9], [-48, 1], [18, -6], [11, -12], [33, -3], [38, 6], [-29, -2], [5, -6], [-45, -3], [8, 6], [-10, -4], [-6, 6], [-34, 5], [37, 2], [6, 12], [14, -10], [-3, 9], [-8, -4], [-10, 0], [0, -14], [-6, 12], [0, 3], [-21, 1], [45, 0], [23, 6], [29, 9], [-7, -1], [-34, -9], [-33, 0], [9, 7], [19, 5], [11, 12], [20, -4], [-36, -5], [-24, -6], [18, 6], [-6, -18], [34, -4], [39, -5], [-4, 3], [-30, 0], [10, -13], [-46, -4], [-35, -10], [24, 3], [-21, 0], [-44, -2], [10, 2], [24, -2], [34, 4], [-2, -14], [-9, 12], [-24, -2], [-33, 7], [-25, 3], [32, -1], [-2, -5], [28, 8], [0, 6], [23, -4], [-2, 20], [-60, 1], [2, 17], [24, -11], [-12, -12], [-9, -3], [-40, -6], [-3, 10], [-12, 0], [-17, 10], [-18, -9], [-63, -2], [-24, -2], [12, -12], [12, 19], [6, -12], [-20, 8], [23, 5], [-7, 20], [27, 1], [6, 0], [5, 18], [-2, 23], [6, 4], [16, 1], [-16, -6], [-2, -11], [-24, -5], [7, 4], [-1, 2], [-52, 2], [-3, 12], [-36, 10], [29, -9], [-9, -2], [17, -1], [-27, -3], [42, -2], [-25, -13], [-33, -2], [24, -6], [1, -4], [-18, -18], [-18, 9], [-28, 6], [22, 4], [-14, 1], [-52, 6], [-16, -9], [9, -21], [24, -14], [6, -14], [8, 2], [0, 19], [-48, 4], [-3, -3], [5, 3], [12, 3], [32, 5], [-36, 4], [-12, -3], [-40, -9], [-23, 2], [5, -15], [38, -6], [-2, 2], [1, -1], [-4, 8], [-39, 4], [9, 10], [20, -10], [-8, -11], [-22, 6], [52, 4], [-30, -14], [-22, 9], [-8, 4], [32, -3], [38, 8], [-13, 3], [20, 8], [-3, -6], [11, 11], [18, -12], [25, -11], [54, 1], [-21, 3], [-20, -16], [-24, 12], [11, -7], [38, -12], [41, 9], [0, 12], [-48, 4], [1, 16], [-24, -2], [6, 10], [-24, 9], [8, -21], [52, 10], [-24, -2], [8, -10], [-70, -4], [-9, 0], [-13, 18], [52, 10], [-23, -13], [-22, 14], [-9, 18], [20, -3], [-2, 7], [22, 8], [-58, 6], [-6, -9], [25, -10], [-8, -2], [13, 11], [18, -2], [6, -11], [66, 6], [5, -10], [-24, 10], [63, 6], [-30, 7], [-46, -4], [10, 17], [3, -9], [-13, 12], [15, -15], [30, -17], [-3, -5], [37, -4], [-14, -11], [39, -15], [-69, 4], [24, 0], [29, -16], [-18, -2], [-73, 0], [43, 13], [6, 0], [-19, 21], [18, 7], [54, -9], [6, -20], [35, 0], [4, 1], [-44, 13], [-22, -4], [7, 16], [-10, 8], [-16, -6], [-35, 1], [34, 13], [-52, 8], [19, -7], [9, -26], [63, -3], [-4, 24], [44, 6], [2, -19], [56, 0], [9, -6], [-6, -3], [18, -8], [-16, -24], [63, 3], [-44, 4], [1, 17], [-62, -5], [27, 13], [-37, 5], [26, -1], [34, -13], [-43, 2], [14, 9], [-3, 21], [-19, -18], [-26, 14], [-30, 1], [-12, 18], [-60, 7], [15, 3], [66, 3], [7, -1], [0, -9], [50, -9], [-36, -2], [-26, -17], [-14, 23], [-36, -9], [60, 1], [6, 27], [-41, 8], [30, -6], [14, -19], [-10, 0], [32, 6], [14, -19], [7, -1], [-3, -9], [76, 5], [50, -4], [-16, 20], [-8, 20], [-7, -12], [4, -26], [3, 18], [-58, -6], [-31, 5], [-50, -5], [-22, -4], [-30, 7], [3, 0], [-10, -1], [24, 13], [68, 8], [-38, -8], [-11, -26], [-13, 3], [-27, 9]]
